/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "mySolidParticleCloud.H"
#include "fvMesh.H"
#include "volFields.H"
#include "interpolationCellPoint.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineParticleTypeNameAndDebug(solidParticle, 0);
    defineTemplateTypeNameAndDebug(Cloud<solidParticle>, 0);
};

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::solidParticleCloud::solidParticleCloud
(
    const fvMesh& mesh,
    const word& cloudName
)
:
    Cloud<solidParticle>(mesh, cloudName, false),
 //const volVectorField& U = mesh_.lookupObject<const volVectorField>("U");
    mesh_(mesh),
//runTime_((mesh_.lookupObject<const volVectorField>("U")).time()),
    particleProperties_
    (
        IOobject
        (
            "particleProperties",
            mesh_.time().constant(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    ),
    rhop_(dimensionedScalar(particleProperties_.lookup("rhop")).value()),
    e_(dimensionedScalar(particleProperties_.lookup("e")).value()),
    mu_(dimensionedScalar(particleProperties_.lookup("mu")).value()),
    nbInjByDt_(dimensionedScalar(particleProperties_.lookup("nbInjByDt")).value()),
    center_(dimensionedVector(particleProperties_.lookup("center")).value()),
    r0_(dimensionedVector(particleProperties_.lookup("r0")).value()),
    d_(dimensionedScalar(particleProperties_.lookup("d")).value()),
    vel_(dimensionedVector(particleProperties_.lookup("vel")).value()),
    velprime_(dimensionedScalar(particleProperties_.lookup("velprime")).value()),
    tInjStart_(dimensionedScalar(particleProperties_.lookup("tInjStart")).value()),
    tInjEnd_(dimensionedScalar(particleProperties_.lookup("tInjEnd")).value()),
    random_(666),
    nbPVp_(mesh_.nCells(), 0.0)
{
    solidParticle::readFields(*this);
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::solidParticleCloud::move(const dimensionedVector& g)
{
    const volScalarField& rho = mesh_.lookupObject<const volScalarField>("rho");
    const volVectorField& U = mesh_.lookupObject<const volVectorField>("U");
    const volScalarField& nu = mesh_.lookupObject<const volScalarField>("nu");
   
    interpolationCellPoint<scalar> rhoInterp(rho);
    interpolationCellPoint<vector> UInterp(U);
    interpolationCellPoint<scalar> nuInterp(nu);
    nbPVp_ = 0.0;
    solidParticle::trackData td(*this, rhoInterp, UInterp, nuInterp, g.value());

    Cloud<solidParticle>::move(td);

if(mesh_.time().value()>td.spc().tInjStart_ && mesh_.time().value()<td.spc().tInjEnd_) {
    this->inject(td);
}
}

void Foam::solidParticleCloud::inject(solidParticle::trackData &td) {
        for (label iterP=1; iterP<=td.spc().nbInjByDt(); iterP++) {
          vector tmp=(random().vector01()- vector(0.5,0.5,0.5))*2;
          vector center=td.spc().center();
          vector r0=td.spc().r0();
          scalar posx=tmp.x()*r0.x();
          scalar posy=tmp.y()*r0.y();
          scalar posz=tmp.z()*r0.z();
          vector pos=center+vector(posx,posy,posz);
          vector tmpv=vector(random().GaussNormal(),random().GaussNormal(),random().GaussNormal())/sqrt(3.);
          vector vel=tmpv*td.spc().velprime()+td.spc().vel();
          label cellI=mesh_.findCell(pos);

          if(cellI>=0) {
              solidParticle* ptr= new solidParticle(*this,pos,cellI,td.spc().d(),vel);
              Cloud<solidParticle>::addParticle(ptr);
          }
       }
}


void Foam::solidParticleCloud::writeFields() const
{
    solidParticle::writeFields(*this);
}


// ************************************************************************* //
